import React, { useEffect, useState } from 'react';
import Pagination from '@material-ui/lab/Pagination';
import TextField from '@material-ui/core/TextField';
import DataTable from '../components/DataTable';
import Grid from '@material-ui/core/Grid';
import Toolbar from '@material-ui/core/Toolbar';
import Autocomplete from '@material-ui/lab/Autocomplete';
import { useAuth0 } from '../react-auth0-spa';
import { useType } from '../utils/helpers';

const queryBuilder = ({ max = 15, skip = 0, filter }) => {
  return `q={${filter ? `"year": ${filter}` : ''}}&max=${max}&skip=${skip}`;
};

const restTypesMapper = {
  '': ['sunshine', 'snow', 'precipitation'],
  air: ['wind-speed', 'air-pressure', 'air-moist'],
  temp: ['temp-avg', 'temp-min', 'temp-max'],
};

const HomePage = () => {
  const type = useType();
  const [years, setYears] = useState([]);
  const [data, setData] = useState([]);
  const { apiSun, apiTemp, apiAir } = useAuth0();
  const [{ max, skip, count, filterValue }, setPagination] = useState({
    max: 15,
    skip: 0,
    count: 0,
    filterValue: '',
  });
  const [initialized, setInitialized] = useState(false);
  const [{ resource, rest }, setRestApi] = useState({
    resource: apiSun,
    rest: 'sunshine',
  });

  React.useEffect(() => {
    if (!type) {
      setRestApi({
        resource: apiSun,
        rest: 'sunshine',
      });
    } else if (type === 'air') {
      setRestApi({
        resource: apiAir,
        rest: 'wind-speed',
      });
    } else if (type === 'temp') {
      setRestApi({
        resource: apiTemp,
        rest: 'temp-avg',
      });
    }
  }, [type]);
  const onFilter = (e, filter) => {
    setPagination((prevPagination) => ({
      ...prevPagination,
      filterValue: filter,
    }));
  };

  const handlePagination = (_event, page) => {
    setInitialized(false);
    setPagination((prevPagination) => ({
      ...prevPagination,
      skip: (page - 1) * prevPagination.max,
    }));
  };
  useEffect(() => {
    setInitialized(false);
    Promise.all([
      resource.get(`rest/${rest}?${queryBuilder({ max, skip, filter: filterValue })}&totals=true`).then(({ data, totals }) => {
        setData(data);
        setPagination((prevPagination) => ({ ...prevPagination, count: totals.total }));
      }),
      resource.get(`rest/${rest}?q={}&h={"$fields": {"year": 1}}`).then((data) => setYears([...new Set(data.map(({ year }) => `${year}`))])),
    ]).then(() => setInitialized(true));
  }, [skip, filterValue, resource, rest]);

  // const addMessage = () => apiSun.updateData('professors.messages', { message: 'Hola' }, 'add');
  // const replaceMessage = () => apiSun.updateData('professors.messages', { message: 'Hola' }, 'replace');

  return (
    <div>
      <h1>Home page</h1>
      <Grid container spacing={3} justify="space-between">
        <Grid item>
          <Autocomplete
            style={{ minWidth: '300px' }}
            options={restTypesMapper[type || '']}
            id="rest-type-picker"
            value={rest}
            onChange={(e, filter) => setRestApi(({ resource }) => ({ resource, rest: filter }))}
            disabled={!initialized}
            renderInput={(params) => <TextField {...params} label="Select rest type" margin="normal" />}
          />
        </Grid>
      </Grid>

      <Toolbar>
        <Grid container spacing={3} justify="space-between">
          <Grid item>
            <Autocomplete
              style={{ minWidth: '150px' }}
              options={years}
              id="year-picker"
              value={filterValue}
              onChange={onFilter}
              disabled={!initialized}
              renderInput={(params) => <TextField {...params} label="Select year" margin="normal" />}
            />
          </Grid>
          <Grid item>
            <Pagination
              style={{ marginTop: '12px' }}
              count={Math.ceil(count / max)}
              page={skip / max + 1}
              onChange={handlePagination}
              variant="outlined"
              shape="rounded"
              showFirstButton
              showLastButton
            />
          </Grid>
        </Grid>
      </Toolbar>
      <DataTable rows={data} initialized={initialized} />
    </div>
  );
};

export default HomePage;
